<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

if ( ! class_exists( 'Ivole_Admin_Import' ) ):

    class Ivole_Admin_Import {

        protected static $background_importer;

        public static $columns = array(
            'review_content',
            'review_score',
            'date',
            'product_id',
            'display_name',
            'email'
        );

        public function __construct() {
            add_action( 'init', array( 'Ivole_Admin_Import', 'init_background_importer' ) );
            add_action( 'wp_ajax_ivole_import_upload_csv', array( $this, 'handle_upload' ) );
            add_action( 'wp_ajax_ivole_check_import_progress', array( $this, 'check_import_progress' ) );
            add_action( 'wp_ajax_ivole_cancel_import', array( $this, 'cancel_import' ) );
        }

        /**
         * Receives a CSV file via ajax, validates it and counts reviews
         */
        public function handle_upload() {
            if ( ! current_user_can( 'upload_files' ) ) {
                echo wp_json_encode(
                    array(
                        'success' => false,
                        'data'    => array(
                            'message' => __( 'Permission denied', IVOLE_TEXT_DOMAIN )
                        ),
                    )
                );
                wp_die();
            }

            if ( ! isset( $_FILES['file'] ) || ! is_array( $_FILES['file'] ) ) {
                echo wp_json_encode(
                    array(
                        'success' => false,
                        'data'    => array(
                            'message' => __( 'No file was uploaded', IVOLE_TEXT_DOMAIN )
                        ),
                    )
                );
                wp_die();
            }

            $wp_filetype = wp_check_filetype_and_ext( $_FILES['file']['tmp_name'], $_FILES['file']['name'] );

		    if ( $wp_filetype['ext'] !== 'csv' ) {
                echo wp_json_encode(
                    array(
                        'success' => false,
                        'data'    => array(
                            'message'  => __( 'The uploaded file is not a valid CSV file', IVOLE_TEXT_DOMAIN ),
                            'filename' => $_FILES['file']['name'],
                        )
                    )
                );
                wp_die();
            }

            $file_data = wp_handle_upload( $_FILES['file'], array(
                'action' => 'ivole_import_upload_csv'
            ) );

            if ( isset( $file_data['error'] ) ) {
                echo wp_json_encode(
                    array(
                        'success' => false,
                        'data'    => array(
                            'message'  => $file_data['error'],
                            'filename' => $_FILES['file']['name'],
                        )
                    )
                );
                wp_die();
            }

            $file_stats = $this->validate_csv_file( $file_data['file'] );

            if ( is_wp_error( $file_stats ) ) {
                echo wp_json_encode(
                    array(
                        'success' => false,
                        'data'    => array(
                            'message'  => $file_stats->get_error_message(),
                            'filename' => $_FILES['file']['name'],
                        )
                    )
                );
                wp_die();
            }

            $progress_id = 'import_progress_' . uniqid();
            $progress = array(
                'status'  => 'importing',
                'started' => current_time( 'timestamp' ),
                'reviews' => array(
                    'total'    => $file_stats['num_reviews'],
                    'imported' => 0,
                    'skipped'  => 0,
                    'errors'   => 0
                )
            );

            set_transient( $progress_id, $progress, WEEK_IN_SECONDS );

            $batch = array(
                'file'        => $file_data['file'],
                'offset'      => $file_stats['offset'],
                'progress_id' => $progress_id
            );

            self::$background_importer->data( $batch );

            self::$background_importer->save()->dispatch();

            echo wp_json_encode(
                array(
                    'success' => true,
                    'data'    => array(
                        'file_path'   => $file_data['file'],
                        'num_rows'    => $file_stats['num_reviews'],
                        'progress_id' => $progress_id
                    )
                )
            );
            wp_die();
        }

        /**
         * Ensures the first line of the csv is formatted correctly and returns
         * the number of reviews in the file and the offset for the first review
         */
        protected function validate_csv_file( $file_path ) {
            if ( ! is_readable( $file_path ) ) {
                return new WP_Error( 'failed_read_file', __( 'Cannot read CSV file', IVOLE_TEXT_DOMAIN ) );
            }

            $file = fopen( $file_path, 'r' );
            $columns = fgetcsv( $file );

            if ( ! is_array( $columns ) || $columns != self::get_columns() ) {
                fclose( $file );
                return new WP_Error( 'malformed_columns', __( 'The CSV file contains invalid or missing column headings, please refer to the template in step 1', IVOLE_TEXT_DOMAIN ) );
            }

            $offset = ftell( $file );

            $num_reviews = 0;
            while ( ( $row = fgetcsv( $file ) ) !== false ) {
                $num_reviews++;
            }

            fclose( $file );

            if ( $num_reviews < 1 ) {
                return new WP_Error( 'no_reviews', __( 'The CSV file contains no reviews', IVOLE_TEXT_DOMAIN ) );
            }

            return array(
                'offset'      => $offset,
                'num_reviews' => $num_reviews
            );
        }

        public function check_import_progress() {
            $progress_id = $_POST['progress_id'];
            $progress = get_transient( $progress_id );

            wp_send_json( $progress, 200 );
            wp_die();
        }

        public function cancel_import() {
            $progress_id = $_POST['progress_id'];

            set_transient( 'cancel' . $progress_id, true, WEEK_IN_SECONDS );

            wp_die();
        }

        /**
         * Initialize the background importer process
         */
        public static function init_background_importer() {
            if ( ! class_exists( 'Ivole_Background_Importer' ) ) {
                include_once dirname( __FILE__ ) . '/class-ivole-background-importer.php';
            }

            self::$background_importer = new Ivole_Background_Importer();
        }

        public static function get_columns() {
            return self::$columns;
        }
    }

endif;
